*:*********************************************************************
*:
*:        Program: CHGCHR
*:
*:         System: Change characters in a file
*:         Author: John Wright
*:      Copyright (c) 1995, John Wright
*:
*:*********************************************************************
* 05/07/95 - Replacement for NULLSTRP program with more functionality.
* 05/07/95 - Oops!  Small bug found after initial version sent out.

#INCLUDE fileio.hdr
#INCLUDE gauge.hdr
#INCLUDE io.hdr
#INCLUDE screen.hdr
#INCLUDE string.hdr
#INCLUDE system.hdr

#PRAGMA W_FUNC_PROC-

* Define variables used by system
VARDEF
  CHAR      cParams
  CHAR      cFileFrom
  CHAR      cFileTo
  CHAR(250) cNew
  CHAR(250) cOld
  CHAR(1)   cLetter
  BYTE      OldText[250] BASED cOld
  INT       niFrom
  INT       niTo
  INT       niLength
  INT       niLetter
  INT       niLoop
  INT       niSpot
  LOGICAL   lRemove
  LONG      nlChanged
  LONG      nlPercent
  LONG      nlTotal
  UINT      nuInput
  UINT      nuOutput
  UINT      nuGauge      && gauge handle
ENDDEF

*!*********************************************************************
*!
*!      Procedure: FORCE_MAIN
*!
*!*********************************************************************
PROCEDURE Force_main
PARAMETERS CHAR(127) cCmdLine

cParams  := UPPER( LTRIM( cCmdLine ) ) + " "
niLength := LEN( cParams )

IF niLength <= 1 .OR. "?" $ cParams
   ?"Ŀ"
   ?" CHGCHR - 1.1                          Change character utility "
   ?"Ĵ"
   ?" Change a character by replacing it with another.               "
   ?"                                                                "
   ?" CHGCHR is written in Xbase and compiled by Force.  The program "
   ?" was initially created to remove NULL characters from a file by "
   ?" replacing them with spaces.  CHGCHR can read a file with NULL  "
   ?" characters if they are being replaced or removed.  The program "
   ?" cannot write NULL characters back to a file.  NULLs are used   "
   ?" by the compiler to denote the end of a character variable.     "
   ?"                                                                "
   ?" This is a FREE program written by John Wright [70534,2330]     "
   ?"Ĵ"
   ?" Syntax:  CHGCHR <FromFile> <ToFile> [-###] [+###] [/NOT]       "
   ?"                                                                "
   ?" Option:  -###  ASCII character to remove  (default: 000)       "
   ?"          +###  ASCII character to replace (default: 032)       "
   ?"          /NOT  Do NOT replace the removed character.           "
   ?"                                                                "
   ?" NOTE:    From filename and To filename are REQUIRED!           "
   ?"          Optional parameters MUST contain three digits!        "
   ?""
   ?" "
   QUIT
ENDIF

niSpot := AT( " ", cParams )
IF niSpot > 0
   cFileFrom := SUBSTR( cParams, 1, niSpot-1 )
   cOld      := SUBSTR( cParams, niSpot+1, niLength - niSpot )
   cParams   := cOld
   niLength  := LEN( cParams )
ENDIF

IF .NOT. FB_OPEN( nuInput, cFileFrom, &B_READ )
   ?"ERROR:  Cannot open file => "+cFileFrom
   ?""
   QUIT
ENDIF

* Get file size and return to start of file
nlTotal := FB_SEEK( nuInput, 0, &FB_END )
FB_SEEK( nuInput, 0, &FB_BEGIN )

niSpot := AT( " ", cParams )
IF niSpot > 0
   cFileTo  := SUBSTR( cParams, 1, niSpot-1 )
ENDIF

IF EXIST( cFileTo )
   ERASE cFileTo
ENDIF

IF .NOT. FB_OPEN( nuOutput, cFileTo, &B_CWRITE )
   FB_CLOSE( nuInput )
   ?"ERROR:  Cannot create file => "+cFileTo
   ?""
   QUIT
ENDIF

* look for Remove character parameter
niSpot := AT( "-", cParams )
IF niSpot > 0
   niFrom := I_VAL( SUBSTR( cParams, niSpot+1, 3 ) )
ELSE
   niFrom := 000
ENDIF

* look for Insert character parameter
niSpot := AT( "+", cParams )
IF niSpot > 0
   niTo := I_VAL( SUBSTR( cParams, niSpot+1, 3 ) )
ELSE
   niTo := 032
ENDIF

* look for NOT parameter
lRemove := ( AT( "/NOT", cParams ) > 0 )

Clear
Cursor_Off()

@ 06,30 ?? "From: "+cFileFrom
@ 08,30 ?? "  To: "+cFileTo

IF lRemove
   @ 10,28 ?? "Remove: CHR("+LTRIM(STR(niFrom,3,0))+")"
ELSE
   @ 10,28 ?? "Change: CHR("+LTRIM(STR(niFrom,3,0))+") to CHR("+LTRIM(STR(niTo,3,0))+")"
ENDIF

@ 12,30 ?? "Done: 0"

nuGauge := GaugeNew( 15, 15, 17, 65, 0x07, 0x70, CHR(219) )
GaugeDisplay( nuGauge )

nlChanged := 0
nlPercent := 0

DO WHILE .NOT. FB_EOF( nuInput )

   IF nlTotal - nlPercent > 250
      niLength := 250
   ELSE
      niLength := nlTotal - nlPercent
   ENDIF

   cOld := ""
   FB_READ( nuInput, cOld, niLength )

   cNew   := ""
   niLoop := 0
   DO WHILE niLoop < niLength
      * Had to use BYTE/BASED array in order to handle CHR(0) values
      * in a character string.
      niLetter := OldText[ niLoop ]
      IF niLetter == niFrom
         IF lRemove
            cLetter := ""
         ELSE
            cLetter := CHR( niTo )
         ENDIF
         nlChanged ++
         @ 12,36 ?? LTRIM( STR( nlChanged, 12, 0 ) )
      ELSE
         cLetter := CHR( niLetter )
      ENDIF
      cNew += cLetter
      nlPercent ++
      niLoop ++
   ENDDO

   GaugeUpdate( nuGauge, nlPercent, nlTotal )

   FB_WRITE( nuOutput, cNew, LEN(cNew) )

ENDDO

Cursor_On()
GaugeClear()

FB_CLOSE( nuInput )
FB_CLOSE( nuOutput )

@ 22,00 SAY ""
?""

QUIT

ENDPRO

*: EOF: CHGCHR.PRG
